module GetDirection( clock, KeyX, KeyY, HSync, VSync, iDir, 
			iKeyTmpX, PosKeyX, oPosKeyX, iKeyTmpXZero, iKeyTmpXMax);

	input clock, KeyX, KeyY, HSync, VSync;

///////////////////////////////////////////
	reg qKeyX, qKeyX2, qKeyY, qKeyY2;
	always @(posedge clock)
	begin
		qKeyX2 <= ~KeyX;		// active low inputs, and get rid of metastability problems
		qKeyX  <= qKeyX2;
		qKeyY2 <= ~KeyY;
		qKeyY  <= qKeyY2;
	end
	
	output [4:0] iKeyTmpX;
	wire [4:0] iKeyTmpY;
	
	output PosKeyX, oPosKeyX, iKeyTmpXZero, iKeyTmpXMax;
	reg    PosKeyX, oPosKeyX, iKeyTmpXZero, iKeyTmpXMax;
	reg    PosKeyY, oPosKeyY, iKeyTmpYZero, iKeyTmpYMax;

// counters for X and Y keys, won't count below 0 and won't count beyond 31
	lpm_counter CntiKeyTmpX( .clock(clock), .q(iKeyTmpX), .updown(qKeyX), .cnt_en(HSync & ((~iKeyTmpXZero & ~qKeyX) | (~iKeyTmpXMax & qKeyX))) );
	defparam CntiKeyTmpX.LPM_WIDTH=5;
	lpm_counter CntiKeyTmpY( .clock(clock), .q(iKeyTmpY), .updown(qKeyY), .cnt_en(HSync & ((~iKeyTmpYZero & ~qKeyY) | (~iKeyTmpYMax & qKeyY))) );
	defparam CntiKeyTmpY.LPM_WIDTH=5;

	always @(posedge clock)
	begin
		iKeyTmpXZero <= iKeyTmpX==0;
		iKeyTmpXMax <= iKeyTmpX==31;
		
		PosKeyX <= (PosKeyX | iKeyTmpXMax) & ~iKeyTmpXZero;
		oPosKeyX <= PosKeyX;
	end
	
	always @(posedge clock)
	begin
		iKeyTmpYZero <= iKeyTmpY==0;
		iKeyTmpYMax <= iKeyTmpY==31;
		
		PosKeyY <= (PosKeyY | iKeyTmpYMax) & ~iKeyTmpYZero;
		oPosKeyY <= PosKeyY;
	end

///////////////////////////////////////////
// make sure you turn only once per video frame	

	reg ChangeDirX, ChangeDirY, TurnX, TurnY;
	always @(posedge clock)
	begin
		TurnX <= ((PosKeyX & ~oPosKeyX) | TurnX) & ~VSync;
		TurnY <= ((PosKeyY & ~oPosKeyY) | TurnY) & ~VSync;
		
		ChangeDirX <= VSync &  TurnX;
		ChangeDirY <= VSync & ~TurnX & TurnY;
	end
	
	output [1:0] iDir;
	lpm_counter CntiDir( .clock(clock), .q(iDir), .updown(ChangeDirX), .cnt_en(ChangeDirX | ChangeDirY) );
	defparam CntiDir.LPM_WIDTH=2;

endmodule

