module moveworm( clock, HSync, VSync, t_VB_MAP, addqx, addqy, weVB, RasterOut, DeadWorm,
					EndChangeHead, EndChangeTail, XPos, YPos, MapX_q, MapY_q, 
					KeyX, KeyY, NewColision, 
					HeadX, HeadY, iHead, FindHead );
	input clock, HSync, VSync, t_VB_MAP, weVB, KeyX, KeyY, NewColision, DeadWorm;
	input [7:0] addqx, addqy, MapX_q, MapY_q;
	input [3:0] RasterOut;
	input [9:0] XPos, YPos;
	
///////////////////////////////////////////
	reg VSyncYet, InitCnt;
	always @(posedge clock)
	begin
		VSyncYet <= VSyncYet | HSync;
		InitCnt <= ~VSyncYet & HSync;	// get a one time only pulse
	end

///////////////////////////////////////////
	wire [1:0] iDir;
	wire enCntXHead;
	output [7:0] HeadX;
	lpm_counter CntHeadX( .clock(clock), .q(HeadX), .sset(InitCnt), .updown(~iDir[1]), .cnt_en(enCntXHead) );
	defparam CntHeadX.LPM_WIDTH=8, CntHeadX.LPM_SVALUE=32;

	wire enCntYHead;
	output [7:0] HeadY;
	lpm_counter CntHeadY( .clock(clock), .q(HeadY), .sset(InitCnt), .updown( iDir[1]), .cnt_en(enCntYHead) );
	defparam CntHeadY.LPM_WIDTH=8, CntHeadY.LPM_SVALUE=50;

	wire enCntHead;
	output [6:0] iHead;
	lpm_counter CntiHead( .clock(clock), .q(iHead), .sset(InitCnt), .cnt_en(enCntHead) );
	defparam CntiHead.LPM_WIDTH=7, CntiHead.LPM_SVALUE=4;
	wire [7:0] addHead;
	assign addHead = {1'b1, iHead};

	wire enCntTail;
	wire [6:0] iTail;
	lpm_counter CntiTail( .clock(clock), .q(iTail), .cnt_en(enCntTail) );
	defparam CntiTail.LPM_WIDTH=7;
	wire [7:0] addTail;
	assign addTail= {1'b1, iTail};

///////////////////////////////////////////
	wire [7:0] iFrame;
	lpm_counter CntiFrame( .clock(clock), .q(iFrame), .cnt_en(VSync) );
	defparam CntiFrame.LPM_WIDTH=8;

	reg dosomethingVS, dosomething, ChangeHead, ChangeTail, BiggerWorm, MayChangeTail;
	always @(posedge clock)
	begin
//		dosomethingVS <= iFrame[0]==0 && VSync;	// for a slow worm
		dosomethingVS <= VSync;
		dosomething <= dosomethingVS;
//		ChangeHead <= (ChangeHead | (dosomethingVS & ~iFrame[1] & ~iFrame[0])) & ~EndChangeHead;
		ChangeHead <= (ChangeHead | (dosomethingVS & ~iFrame[0])) & ~EndChangeHead;

//		MayChangeTail = (dosomethingVS & iFrame[4] & ~iFrame[0] & ~DeadWorm) | (dosomethingVS & DeadWorm);
		MayChangeTail = (dosomethingVS & ~DeadWorm & iFrame[3]) | (dosomethingVS & DeadWorm);
		BiggerWorm <= (NewColision | BiggerWorm) & ~MayChangeTail;
		ChangeTail <= (ChangeTail | (MayChangeTail & ~BiggerWorm)) & ~EndChangeTail;
	end

///////////////////////////////////////////
	output EndChangeHead, EndChangeTail;
	GetDirection gd( .clock(clock), .KeyX(KeyX), .KeyY(KeyY), .HSync(HSync), .VSync(VSync), .iDir(iDir) );

	assign enCntHead = dosomething & ChangeHead;// & ~DeadWorm;
	assign enCntXHead = enCntHead & ~iDir[0];
	assign enCntYHead = enCntHead &  iDir[0];

//	assign enCntTail = dosomething & ChangeTail;
	assign enCntTail = EndChangeTail;

	output FindHead;
	reg FindHead;
	always @(posedge clock)
	begin
		FindHead <= (addqx==addHead) & ~weVB & ~t_VB_MAP & ~DeadWorm;
	end
	assign EndChangeHead = ChangeHead && FindHead;
	
	reg FindTail;
	always @(posedge clock)
	begin
		FindTail <= (addqx==addTail) & ~weVB & ~t_VB_MAP;
	end
	assign EndChangeTail = ChangeTail && FindTail;

endmodule

