module RasterDisplay( clock, HSync, VSync, XPos, YPos, RasterOut, HeadX, HeadY, EndChangeHead, EndChangeTail, DeadWorm, 
			MapX_q, MapY_q, VB_q, VB_d, t_VB_MAP, addqx, addqy, addVB, weVB, GoodMapY,
			Colision, ColisionFound, EndColision, NewColision, ColisionX, ColisionY, weMAP, MapX_d, MapY_d);

	input clock, HSync, VSync;
	input [9:0] XPos, YPos;	// [0..639], [0..479]
	input [7:0] HeadX, HeadY;
	input EndChangeHead, EndChangeTail;
	output [3:0] RasterOut;
	reg [3:0] RasterOut;

	output GoodMapY;
	reg GoodMapY;
	output [7:0] MapX_q, MapY_q, VB_q;
	output [7:0] VB_d;

///////////////////////////////////////////
	output weVB, t_VB_MAP;
	output [7:0] addqx;

	reg [10:0] qx;
	lpm_counter CountX( .clock(clock), .q(qx), .sclr(HSync & YPos[0]) );
	defparam CountX.LPM_WIDTH = 11;

	assign weVB = qx[0];
	assign t_VB_MAP = qx[1];
//	assign addqx = qx[9:2];
	assign addqx = { qx[9] & ~qx[10], qx[8:2] };	// scan the worm map (addqx[7]=1) only once

///////////////////////////////////////////
	wire [7:0] next_addqy;
	lpm_add_sub addqy_cnt( .dataa(YPos[9:2]), .datab(1), .result(next_addqy) );
	defparam addqy_cnt.lpm_width=8, addqy_cnt.ONE_INPUT_IS_CONSTANT="YES";

	output [7:0] addqy;
	reg [7:0] addqy;
	always @(posedge clock)
	begin
		addqy <= next_addqy;	//YPos[9:2] + 1;
		GoodMapY <= (addqy[6:0]==MapY_q[6:0]);
	end

///////////////////////////////////////////
	output [7:0] MapX_d, MapY_d;
	output weMAP;
	
	lpm_ram_dq MapX( .data(MapX_d), .address(addqx), .q(MapX_q), .inclock(clock), .outclock(clock), .we(weMAP) );
	defparam MapX.LPM_WIDTH=8, MapX.LPM_WIDTHAD=8, MapX.LPM_OUTDATA="REGISTERED", MapX.LPM_FILE="MapX.mif";
	lpm_ram_dq MapY( .data(MapY_d), .address(addqx), .q(MapY_q), .inclock(clock), .outclock(clock), .we(weMAP) );
	defparam MapY.LPM_WIDTH=8, MapY.LPM_WIDTHAD=8, MapY.LPM_OUTDATA="REGISTERED", MapY.LPM_FILE="MapY.mif";

///////////////////////////////////////////
	output [7:0] addVB;
	reg [7:0] addVB;

	always
	begin
		if( t_VB_MAP==0 )
			addVB <= XPos[9:2];
		else
			addVB <= MapX_q;
	end

	lpm_ram_dq VB( .data(VB_d), .address(addVB), .q(VB_q), .inclock(clock), .we(weVB) );
	defparam VB.LPM_WIDTH=8, VB.LPM_WIDTHAD=8, VB.LPM_OUTDATA="UNREGISTERED";

///////////////////////////////////////////
	reg WallX, WallY;

	always @(posedge clock)
	begin
		WallX <= XPos[9:2]==0 || XPos[9:2]==159;
		WallY <= addqy==0 || addqy==119;
	end
	
///////////////////////////////////////////
	wire [3:0] VB_or;
	wire [7:0] VB_and;
	wire BlockIsHere = t_VB_MAP & ~addqx[7] & GoodMapY;
	wire  WormIsHere = t_VB_MAP &  addqx[7] & GoodMapY;

	assign VB_or = {(BlockIsHere &  MapY_q[7]), 								// bit 3: good blocks
				    (BlockIsHere & ~MapY_q[7]) | (~t_VB_MAP & (WallX | WallY)),	// bit 2: bad blocks + walls
					(WormIsHere) & ((VB_q[0] & YPos[2]) | (VB_q[4] & ~YPos[2])),// bit 1: worm hits himself
					(WormIsHere)};												// bit 0: worm
//ColisionFound & ~t_VB_MAP & ((ColisionX==XPos[9:2]) | (ColisionY==YPos[8:2]))	// to draw a debug colision cross

	assign VB_and= { {4{YPos[2] | YPos[1]}}, {4{~YPos[2] | YPos[1]}} };
	assign VB_d = (VB_q | {VB_or & {4{~YPos[2]}},VB_or & {4{YPos[2]}}}) & VB_and;
	
///////////////////////////////////////////
	output Colision, ColisionFound, EndColision, NewColision, DeadWorm;
	reg    Colision, ColisionFound, EndColision, DeadWorm;
	output [7:0] ColisionX, ColisionY;
	reg [7:0] ColisionX;
	reg [6:0] ColisionY;
	
	assign NewColision = Colision & ~ColisionFound;
	
	always @(posedge clock)
	begin
		// The worm dies if he hits himself, or he hits a bad block
		DeadWorm <= (RasterOut[1] | (RasterOut[0] & RasterOut[2]) & ~weVB & t_VB_MAP) | DeadWorm;
		//  colision if hehits a good block
		Colision <=  RasterOut[0] & RasterOut[3] & ~weVB & t_VB_MAP;
		
		ColisionFound <= (ColisionFound | Colision) & ~(/*EndColision |*/ VSync );
		if( NewColision )
		begin
			ColisionX <= XPos[9:2];
			ColisionY <= YPos[8:2];
		end
	end

	always @(posedge clock)
	begin
		EndColision <= (MapX_q==ColisionX) & (MapY_q[6:0]==ColisionY) & ~addqx[7] & ColisionFound & ~weVB & t_VB_MAP;
	end
	
	assign MapX_d = HeadX | {8{EndChangeTail | EndColision}};
	assign MapY_d = HeadY | {8{EndChangeTail | EndColision}};

	assign weMAP = EndChangeHead | EndChangeTail | EndColision;
			
///////////////////////////////////////////
	always @(posedge clock)
	begin
		if( weVB & ~t_VB_MAP )
			RasterOut <= (VB_q[3:0] & {4{~YPos[2]}}) | (VB_q[7:4] & {4{YPos[2]}});
	end

endmodule

