module worm(clock, KeyX, KeyY, vga_hsync, vga_vsync, vga_red, vga_green, vga_blue);

	input clock, KeyX, KeyY;
	output vga_red, vga_green, vga_blue, vga_hsync, vga_vsync;

///////////////////////////////////////////
	wire HSync, VSync;
	sync_generator syncVGA( .clock(clock), 
							.HSync(HSync), .VSync(VSync), .vga_h_sync(vga_hsync), .vga_v_sync(vga_vsync) );

///////////////////////////////////////////
	wire [9:0] XPos, YPos;	// [0..639], [0..479]
	lpm_counter CountX( .clock(clock), .q(XPos), .sclr(HSync) );
	defparam CountX.LPM_WIDTH = 10;
	lpm_counter CountY( .clock(clock), .q(YPos), .sset(VSync), .cnt_en(HSync) );
	defparam CountY.LPM_WIDTH = 10, CountY.LPM_SVALUE = 1020;

///////////////////////////////////////////
	wire ValidX = ~XPos[9] | (XPos[8:7]==0);	// XPos[9:7]<5;
	wire ValidY = ~YPos[9] & (YPos[8:5]!=15);	// YPos[9:5]<15;
	reg Valid, Valid1, Valid2;
	reg GridX, GridY;
	always @(posedge clock)
	begin
		Valid <= ValidX & ValidY;
		Valid1<= Valid;
		Valid2<= Valid1;
		GridX <= XPos[3:0]==4 && YPos[3:0]==0;
		GridY <= YPos[3:0]==4 && XPos[3:0]==0;
	end

///////////////////////////////////////////
	wire [3:0] ro;
	wire [7:0] mX_d, mY_d, addqx, addqy;

	RasterDisplay MyRaster( .clock(clock), .HSync(HSync) , .VSync(VSync), .XPos(XPos), .YPos(YPos), .RasterOut(ro), .DeadWorm(DeadWorm),
							.t_VB_MAP(t_VB_MAP), .addqx(addqx), .addqy(addqy), .EndChangeHead(EndChangeHead), .EndChangeTail(EndChangeTail),
							.HeadX(mX_d), .HeadY(mY_d), .weVB(weVB), .MapX_q(MapX_q), .MapY_q(MapY_q),
							.NewColision(NewColision) );

	MoveWorm MyMoveWorm(.clock(clock), .HSync(HSync), .VSync(VSync), .t_VB_MAP(t_VB_MAP), .DeadWorm(DeadWorm),
						.addqx(addqx), .addqy(addqy), .weVB(weVB), .RasterOut(ro), .XPos(XPos), .YPos(YPos), 
						.HeadX(mX_d), .HeadY(mY_d), .EndChangeHead(EndChangeHead), .EndChangeTail(EndChangeTail),
						.KeyX(KeyX), .KeyY(KeyY), .MapX_q(MapX_q), .MapY_q(MapY_q),
						.NewColision(NewColision) );

///////////////////////////////////////////
	assign vga_red  = Valid2 & (ro[2] ^ ro[1]);
	assign vga_green= Valid2 & (ro[3] ^ ro[1]);
	assign vga_blue = Valid2 & (ro[0] | GridX | GridY);
endmodule

